package com.tagtraum.ideajad;

import com.intellij.openapi.actionSystem.*;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.projectRoots.ProjectRootManager;
import com.intellij.openapi.projectRoots.ProjectRootType;
import com.intellij.openapi.ui.Messages;

import javax.swing.*;
import java.util.zip.ZipFile;
import java.util.ArrayList;
import java.io.*;
import java.net.URL;
import java.net.URLDecoder;

/**
 * 
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id:$
 */
public class DecompileAction extends AnAction {

    private static File extractJarDir;
    private String baseDir;
    private static final boolean windows = System.getProperty("os.name").toLowerCase().startsWith("win");

    public DecompileAction() {
        this(null);
    }

    public DecompileAction(String s) {
        this(s, null, null);
    }

    public DecompileAction(String s, String s1, Icon icon) {
        super(s, s1, icon);
        System.out.println("IdeaJad: (c) 2002 tagtraum industries - http://www.tagtraum.com/");
        System.out.println("IdeaJad: Initialized DecompileAction...");
    }

    public void actionPerformed(AnActionEvent event) {
        try {
            String filesToDecompile = null;
            if (getVirtualFile(event).getUrl().startsWith("jar:")) {
                extractJar(getVirtualFile(event), getExtractDir());
                filesToDecompile = "\"" + getExtractDir() + "/**/*.class\"";
            }
            else {
                filesToDecompile = "\"" + getVirtualFile(event).getPresentableUrl() + "\"";
            }
            VirtualFile[] decompilationDirs = getDecompilationDirs(event);
            if (decompilationDirs.length == 0) {
                Messages.showMessageDialog("No decompilation directory found.", "IdeaJad-Plugin", Messages.getErrorIcon());
                return;
            }
            int indexOfChosenDir = 0;
            if (decompilationDirs.length > 1) {
                String[] dirStrings = getStringArray(decompilationDirs);
                indexOfChosenDir = Messages.showChooseDialog("Please choose a target directory:", "IdeaJad-Plugin", Messages.getQuestionIcon(), dirStrings, dirStrings[0]);
            }

            if (indexOfChosenDir >= 0) {
                VirtualFile decompilationDir = decompilationDirs[indexOfChosenDir];
                Jad jad = new Jad(getBaseDir() + (windows ? "/bin/jad.exe" : "/bin/jad"));
                JadOptions options = new JadOptions();
                options.setRestorePackages(true);
                options.setFileExtension("java");
                options.setOutputDirectory("\""+ decompilationDir.getPath() +"\"");
                jad.decompile(options, filesToDecompile);
                ActionManager.getInstance().getAction("Synchronize").actionPerformed(event);
            }
        } catch (IOException ioe) {
            ioe.printStackTrace();
            Messages.showMessageDialog(ioe.toString(), "IdeaJad-Plugin", Messages.getErrorIcon());
        } catch (InterruptedException e) {
            e.printStackTrace();
            Messages.showMessageDialog(e.toString(), "IdeaJad-Plugin", Messages.getErrorIcon());
        }
    }

    private String[] getStringArray(VirtualFile[] decompilationDirs) {
        String[] dirStrings = new String[decompilationDirs.length];
        for (int i=0; i<decompilationDirs.length; i++) {
            dirStrings[i] = decompilationDirs[i].getPresentableUrl();
        }
        return dirStrings;
    }

    private URL getClassLocation(Class klass) {
        return klass.getClassLoader().getResource(klass.getName().replace('.', '/')+".class");
    }
    private synchronized String getBaseDir() throws IOException {
        if (baseDir == null) {
            URL url = getClassLocation(DecompileAction.class);
            String urlString = url.toString();
            url = new URL(urlString.substring("jar:".length(), urlString.indexOf('!')));
            urlString = URLDecoder.decode(url.getFile());
            if (urlString.startsWith("/") && urlString.indexOf(':') == 2) urlString = urlString.substring(1);
            baseDir = new File(urlString).getParentFile().getParent();
        }
        return baseDir;
    }

    private VirtualFile[] getDecompilationDirs(AnActionEvent event) {
        Project project = getProject(event);
        VirtualFile[] classRoots = ProjectRootManager.getInstance(project).getRoots(ProjectRootType.SOURCE);
        java.util.List list = new ArrayList();
        for (int i=0; i<classRoots.length; i++) {
            if (classRoots[i].isDirectory() && classRoots[i].isWritable()) {
                list.add(classRoots[i]);
            }
        }
        return (VirtualFile[])list.toArray(new VirtualFile[list.size()]);
    }

    public void update(AnActionEvent event) {
        VirtualFile vf = getVirtualFile(event);
        if (vf != null) {
            if ("class".equals(vf.getExtension())) event.getPresentation().setEnabled(true);
            else event.getPresentation().setEnabled(false);
        }
        else {
            event.getPresentation().setEnabled(false);
        }
    }

    private void extractJar(VirtualFile jarFile, File destDir) throws IOException {
        ZipFile zipFile = getZipFile(jarFile);
        ZipHelper.extract(zipFile, destDir, true, null);
        zipFile.close();
    }

    private ZipFile getZipFile(VirtualFile vf) throws IOException {
        int idx = 0;
        if ((idx = vf.getPath().indexOf("jar!")) != -1) {
            return new ZipFile(new File(vf.getPath().substring(0, idx+3)), ZipFile.OPEN_READ);
        }
        if ((idx = vf.getPath().indexOf("zip!")) != -1) {
            return new ZipFile(new File(vf.getPath().substring(0, idx+3)), ZipFile.OPEN_READ);
        }
        throw new IOException("Failed to extract zip/jar-file from path: " + vf.getPath());
    }

    private synchronized static File getExtractDir() {
        if (extractJarDir == null) {
            extractJarDir = new File(System.getProperty("user.home") + "/ideajad/");
            extractJarDir.mkdir();
            extractJarDir.deleteOnExit();
        }
        return extractJarDir;
    }

    private VirtualFile getVirtualFile(AnActionEvent event) {
        return (VirtualFile)event.getDataContext().getData(DataConstants.VIRTUAL_FILE);

    }

    private Project getProject(AnActionEvent event) {
        return (Project)event.getDataContext().getData(DataConstants.PROJECT);
    }


}
